@@ -4,7 +4,7 @@ from urllib.parse import urljoin, urlparse
 import os
 import shutil
 import time
-from abstract_webtools import *
+from .managers import *
 from abstract_utilities import *
 def get_abs_path():
     return os.path.abspath(__file__)
