@@ -3,7 +3,11 @@ from .path_utils import *
 from abstract_utilities import make_list
 _LOCK = threading.RLock()
 # ---------------- Registry ---------------- #
-
+def get_video_url(url=None, video_url=None):
+    video_url = url or video_url
+    if video_url:
+        video_url = get_corrected_url(video_url)
+    return video_url
 class infoRegistry(metaclass=SingletonMeta):
     """Thread-safe registry with all video assets stored under ~/videos/<video_id>/ or flat."""
 
@@ -73,8 +77,10 @@ class infoRegistry(metaclass=SingletonMeta):
         return None
 
     def _write_cached_info(self, video_id: str, info: dict) -> str:
-        cache_dir = self.video_root if self.flat_layout else os.path.join(self.video_root, video_id)
-        check_create_logs(f"making this directory == {cache_dir} == _write_cached_info line 77")
+        # Prefer the actual folder that already holds the video
+        cache_dir = info.get("directory") or (
+            self.video_root if self.flat_layout else os.path.join(self.video_root, video_id)
+        )
         os.makedirs(cache_dir, exist_ok=True)
         cache = os.path.join(cache_dir, "info.json")
         get_atomic_write(cache, info)
@@ -103,6 +109,48 @@ class infoRegistry(metaclass=SingletonMeta):
             rec["timestamp"] = time.time()
             self.registry["by_id"][video_id] = rec
             self._save_registry()
+    def edit_info(self, data: dict, url: str | None = None,
+                  video_id: str | None = None, video_path: str | None = None):
+        cur = self.get_video_info(url=url, video_id=video_id, video_path=video_path, force_refresh=False)
+        if not cur:
+            raise RuntimeError("No existing info to edit")
+
+        cur.update(data or {})
+
+        # Stable ID: prefer existing values; never force a new slug unless absolutely needed
+        canonical = (
+            cur.get("video_id")
+            | cur.get("id")
+            | video_id
+            | (get_sha12(url) if url else None)
+        )
+        if not canonical:
+            canonical = generate_video_id(video_path or "video")
+
+        cur["video_id"] = canonical
+
+        # Use the folder that actually contains the mp4
+        video_dir = (
+            cur.get("directory")
+            or (os.path.dirname(os.path.abspath(cur.get("file_path"))) if cur.get("file_path") else None)
+            or (os.path.dirname(os.path.abspath(video_path)) if video_path else None)
+        )
+
+        if video_dir:
+            # Expand schema IN PLACE (no <root>/<video_id> subdir)
+            cur = ensure_standard_paths(cur, video_dir, flat_layout=True)
+            cache_path = os.path.join(video_dir, "info.json")
+        else:
+            # Last resort
+            base = self.video_root if self.flat_layout else os.path.join(self.video_root, canonical)
+            os.makedirs(base, exist_ok=True)
+            cur = ensure_standard_paths(cur, base, flat_layout=self.flat_layout)
+            cache_path = os.path.join(base, "info.json")
+
+        get_atomic_write(cache_path, cur)
+        self._link(canonical, url, cur.get("file_path") or video_path)
+        cur["info_path"] = cache_path
+        return cur
 
     # ---------- main API ----------
 
@@ -148,57 +196,6 @@ class infoRegistry(metaclass=SingletonMeta):
 
         return None
 
-    def edit_info(self, data: dict, url: str | None = None,
-                  video_id: str | None = None, video_path: str | None = None):
-        # Load current
-        cur = self.get_video_info(url=url, video_id=video_id, video_path=video_path, force_refresh=False)
-        if not cur:
-            raise RuntimeError("No existing info to edit")
-
-        # Merge edits
-        cur.update(data or {})
-
-        # --- Canonical, stable video_id (avoid lowercasing fallbacks) ---
-        canonical = (
-            cur.get("video_id")
-            or cur.get("id")
-            or video_id
-            or (get_sha12(url) if url else None)
-        )
-        if not canonical:
-            # last-resort only (try hard to avoid this to prevent new folders)
-            canonical = generate_video_id(video_path or "video")
-
-        cur["video_id"] = canonical
-
-        # --- Anchor schema to the REAL video folder (the one with video.mp4) ---
-        # Prefer explicit directory, else derive from file_path / video_path
-        video_dir = cur.get("directory")
-        if not video_dir:
-            fp = cur.get("file_path") or cur.get("video_path") or video_path
-            if fp:
-                video_dir = os.path.dirname(os.path.abspath(fp))
-
-        # If we know the real folder, expand schema IN PLACE (flat)
-        # This prevents creating a second <video_root>/<video_id> directory.
-        if video_dir:
-            cur = ensure_standard_paths(cur, video_dir, flat_layout=True)
-            cache_path = os.path.join(video_dir, "info.json")
-        else:
-            # Fallback: use registry root with configured layout
-            base = self.video_root if self.flat_layout else os.path.join(self.video_root, canonical)
-            os.makedirs(base, exist_ok=True)
-            cur = ensure_standard_paths(cur, base, flat_layout=self.flat_layout)
-            cache_path = os.path.join(base, "info.json")
-
-        # Write the single canonical info.json next to the video (or in fallback base)
-        get_atomic_write(cache_path, cur)
-
-        # Keep registry links in sync
-        self._link(canonical, url, cur.get("file_path") or video_path)
-
-        cur["info_path"] = cache_path
-        return cur
     def list_cached_videos(self):
         with _LOCK:
             return [
@@ -302,14 +299,17 @@ class VideoDownloader:
                          video_path=None, output_filename=None, ydl_opts=None,
                          registry=None, force_refresh=False,
                          flat_layout: bool = False,video_url =None):
-        video_url = url = get_corrected_url(url or video_url or self.url or self.video_url) 
-
+        video_url = get_video_url(url or video_url or self.url or self.video_url)
+        self.info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
+        info_video_path = self.info.get('video_path')
+        if info_video_path and os.path.isfile(info_video_path) and not force_refresh:
+            self.video_path = info_video_path
+            self.video_id = self.info.get("id")
+            self.video_url = video_url
+            return info
         ydl_opts = ydl_opts or self.ydl_opts or {}
         user_agent = user_agent or self.user_agent
-
         force_refresh = force_refresh or self.force_refresh
-         
-         
         download_directory = download_directory or self.download_directory
         video_extention = video_extention or self.video_extention
         output_filename = output_filename or self.output_filename
@@ -321,8 +321,6 @@ class VideoDownloader:
              filename,ext = os.path.splitext(basename)
              output_filename = filename if filename else output_filename or 'video'
              video_extention = ext if ext else video_extention or '.mp4'
-             
-         
         output_filename = output_filename or 'video'
         download_directory = download_directory  or get_video_root(download_directory)
         flat_layout = flat_layout or self.flat_layout
@@ -376,8 +374,24 @@ class VideoDownloader:
         info = self.registry.get_video_info(video_id=video_id)
         logger.info(f"[VideoDownloader] Stored in registry at {info['video_path']}")
         return info
-
-
+def get_info_from_mgr(mgr):
+    if hasattr(mgr, 'info'):
+        return mgr.info
+    video_path, video_id, video_url = None, None, None
+    if hasattr(mgr, 'video_path'):
+        video_path = mgr.video_path
+    if hasattr(mgr, 'video_id'):
+        video_id = mgr.video_id
+    elif hasattr(mgr, 'id'):
+        video_id = mgr.id
+    if hasattr(mgr, 'video_url'):
+        video_url = mgr.video_url
+    video_info = get_video_info(
+        url=video_id,
+        video_url=video_url,
+        video_path=video_path
+        )
+    return video_info
 def get_videoDownloader(url=None,
                    download_directory=None,
                    user_agent=None,
@@ -390,8 +404,9 @@ def get_videoDownloader(url=None,
                    force_refresh=False,
                    flat_layout: bool = False,
                    video_url=None):
+    video_url = get_video_url(url or video_url)
     videoDownload_mgr = VideoDownloader(
-        url=url,
+        video_url=video_url,
         download_directory=download_directory,
         user_agent=user_agent,
         video_extention=video_extention,
@@ -400,8 +415,7 @@ def get_videoDownloader(url=None,
         output_filename=output_filename,
         ydl_opts=ydl_opts,
         registry=registry,
-        flat_layout=flat_layout,
-        video_url=video_url
+        flat_layout=flat_layout
         )
     return videoDownload_mgr
 def download_video(url=None,
@@ -416,9 +430,9 @@ def download_video(url=None,
                    force_refresh=False,
                    flat_layout: bool = False,
                    video_url=None):
-    
+        video_url = get_video_url(url or video_url)
         videoDownload_mgr = get_videoDownloader(
-            url=url,
+            video_url=video_url,
             download_directory=download_directory,
             user_agent=user_agent,
             video_extention=video_extention,
@@ -427,138 +441,135 @@ def download_video(url=None,
             output_filename=output_filename,
             ydl_opts=ydl_opts,
             registry=registry,
-            flat_layout=flat_layout,
-            video_url=video_url
+            flat_layout=flat_layout
             )
-        return videoDownload_mgr._download_single()
-def get_registryManager(
-    video_directory=None,
-    envPath=None,
-    info_directory=None
-    ):
-    return infoRegistry(video_directory=video_directory, envPath=envPath, info_directory=info_directory)
-def get_video_info(
-    url=None,
-    video_id=None,
-    force_refresh=False,
-    video_directory=None,
-    envPath=None,
-    info_directory=None,
-    video_path=None,
-    video_url=None,
-    download=False
-    ):
-    url = url or video_url
-    registryMgr = get_registryManager(video_directory=video_directory, envPath=envPath, info_directory=info_directory)
-    return registryMgr.get_video_info(url=url, video_id=video_id, force_refresh=force_refresh,video_path=video_path,download=download)
-def get_video_info_spec(
-    key=None,
-    url=None,
-    video_id=None,
-    force_refresh=False,
-    video_directory=None,
-    envPath=None,
-    info_directory=None,
-    video_path=None,
-    video_url=None,
-    download=None
-    ):
-    url = url or video_url
-    video_info = get_video_info(
+        
+        return get_info_from_mgr(videoDownload_mgr)
+
+def get_infoRegistry(video_root=None, flat_layout=None):
+    return infoRegistry(video_root=video_root, flat_layout=flat_layout)
+
+def get_registry_video_root(video_root=None, flat_layout=None):
+    registry_mgr = get_infoRegistry(video_root=video_root, flat_layout=flat_layout)
+    return registry_mgr.video_root
+
+def get_registry_path(video_root=None, flat_layout=None):
+    registry_mgr = get_infoRegistry(video_root=video_root, flat_layout=flat_layout)
+    return registry_mgr.registry_path
+
+def get_video_info(url=None, video_url=None, video_id=None, 
+                        video_path=None, video_root=None,
+                        force_refresh=False, flat_layout=None, download=False):
+    registry_mgr = get_infoRegistry(video_root=video_root, flat_layout=flat_layout)
+    url = get_video_url(url or video_url)
+    video_info = registry_mgr.get_video_info(
         url=url,
         video_id=video_id,
-        force_refresh=force_refresh,
-        video_directory=video_directory,
-        envPath=envPath,
-        info_directory=info_directory,
         video_path=video_path,
-        download=download
+        force_refresh=force_refresh,
+    )
+    if download:
+        video_info = download_video(
+            video_url=video_url,
+            download_directory=video_root,
+            download_video=download,
+            video_path=video_path,
+            flat_layout=flat_layout
+            )
+    return video_info or {}
 
+def get_video_info_spec(url=None, video_url=None, video_id=None, 
+                        video_path=None, video_root=None, key=None,
+                        force_refresh=False, flat_layout=None, download=False):
+    video_url = get_video_url(url or video_url)
+    video_info = get_video_info(
+        video_url=video_url,
+        video_id=video_id,
+        video_path=video_path,
+        force_refresh=force_refresh,
+        video_root=video_root,
+        flat_layout=flat_layout,
+        download=download
     )
     if not key:
         return video_info
-    value = video_info.get(key)
-    if not value:
-         value = make_list(get_any_value(video_info,key) or None)[0]
+    keys = make_list(key)
+    value = None
+    for key in keys:
+        value = video_info.get(key)
+        if not value:
+            values = make_list(get_any_value(video_info, key) or None)
+            value = values[0]
+        if value:
+            break
     return value
+
 def get_video_id(
-    url=None,
-    video_id=None,
-    force_refresh=False,
-    video_directory=None,
-    envPath=None,
-    info_directory=None,
-    video_path=None,
-    video_url=None,
-    download=False
+        url=None,
+        video_url=None,
+        video_id=None,
+        video_path=None,
+        force_refresh=False,
+        download=False,
+        video_root=None,
+        flat_layout=None
     ):
-    url = url or video_url
-    if download:
-        VideoDownloader(url=url)
-    return get_video_info_spec(
-        key='id',
+    info_spec = get_video_info_spec(
         url=url,
+        video_url=video_url,
         video_id=video_id,
-        force_refresh=force_refresh,
-        video_directory=video_directory,
-        envPath=envPath,
-        info_directory=info_directory,
         video_path=video_path,
+        force_refresh=force_refresh,
         download=download,
-        
+        video_root=video_root,
+        flat_layout=flat_layout,
+        key=['id','video_id']
         )
+    return info_spec
 def get_video_title(
-    url=None,
-    video_id=None,
-    force_refresh=False,
-    video_directory=None,
-    envPath=None,
-    info_directory=None,
-    video_path=None,
-    video_url=None,
-    download=False
-    
+        url=None,
+        video_url=None,
+        video_id=None,
+        video_path=None,
+        force_refresh=False,
+        download=False,
+        video_root=None,
+        flat_layout=None
     ):
-    url = url or video_url
-    if download:
-        VideoDownloader(url=url)
-    return get_video_info_spec(
-        key='title',
+    info_spec = get_video_info_spec(
         url=url,
+        video_url=video_url,
         video_id=video_id,
-        force_refresh=force_refresh,
-        video_directory=video_directory,
-        envPath=envPath,
-        info_directory=info_directory,
         video_path=video_path,
-        download=download
+        force_refresh=force_refresh,
+        download=download,
+        video_root=video_root,
+        flat_layout=flat_layout,
+        key=['title']
         )
+    return info_spec
 def get_video_filepath(
-    url=None,
-    video_id=None,
-    force_refresh=False,
-    video_directory=None,
-    envPath=None,
-    info_directory=None,
-    video_path=None,
-    video_url=None,
-    download=False
-    
+        url=None,
+        video_url=None,
+        video_id=None,
+        video_path=None,
+        force_refresh=False,
+        download=False,
+        video_root=None,
+        flat_layout=None
     ):
-    url = url or video_url
-    if download:
-        VideoDownloader(url=url)
-    return get_video_info_spec(
-        key='video_path',
+    info_spec = get_video_info_spec(
         url=url,
+        video_url=video_url,
         video_id=video_id,
-        force_refresh=force_refresh,
-        video_directory=video_directory,
-        envPath=envPath,
-        info_directory=info_directory,
         video_path=video_path,
-        download=download
+        force_refresh=force_refresh,
+        download=download,
+        video_root=video_root,
+        flat_layout=flat_layout,
+        key=['filepath','file_path','video_path','videopath']
         )
+    return info_spec
 
 def get_video_info_from_mgr(video_mgr):
     try:
@@ -567,3 +578,4 @@ def get_video_info_from_mgr(video_mgr):
     except Exception as e:
         print(f"{e}")
         return None
+
